# This is free software released into the public domain (CC0 license).

require 'filepath/filepath.rb'
require 'filepath/filepathlist.rb'
require 'filepath/core_ext/array.rb'
require 'filepath/core_ext/string.rb'

